from typing import List, Optional

from pydantic import BaseModel

from giza_datasets.defillama_constants import DEFILLAMA_SUPPORTED_PROJECTS

sector_tags = ["DeFi"]
application_tags = ["Liquid Staking", "Dexes", "Yield", "Lending", "Yield Aggregator"]
protocol_tags = DEFILLAMA_SUPPORTED_PROJECTS + [
    "Aave-v2",
    "Aave-v3",
    "Compound-v2",
    "Balancer-v1",
    "Balancer-v2",
    "MorphoBlue",
    "Uniswap-v3",
    "Yearn-v2",
]
network_tags = [
    "Ethereum",
    "Arbitrum",
    "Optimism",
    "Avalanche",
    "Base",
    "Gnosis",
    "Polygon",
    "multi-chain",
]
task_tags = [
    "TVL",
    "Token Price",
    "Swap Fees",
    "Liquidity",
    "Borrows & Deposits",
    "Trade Volume",
    "Fees",
    "APY",
    "daily",
    "aggregated",
    "Deposits",
    "Withdraws",
]


class Dataset(BaseModel):
    """
    A Pydantic model representing a Dataset.

    Attributes:
        name (str): The name of the dataset.
        path (str): The path to the dataset.
        description (str): A brief description of the dataset.
        labels Optional[List[str]]: Optional, predefined list of labels associated with the dataset.
        documentation (str): The documentation associated with the dataset.
    """

    name: str
    path: str
    description: str
    tags: Optional[List[str]]
    documentation: str



from rich.console import Console
from rich.table import Table

from giza_datasets.constants import DATASET_HUB


class DatasetsHub:
    """
    Class to manage datasets from the DATASET_HUB.

    Attributes:
        datasets (list): A list of Dataset objects from the DATASET_HUB.

    Methods:
        show(): Prints a table of all datasets in the hub.
        list(): Returns a list of all datasets in the hub.
        get(dataset_name): Returns a Dataset object with the given name.
        describe(dataset_name): Prints a table of details for the given dataset.
    """

    def __init__(self):
        """
        Initializes the DatasetsHub with the datasets from the DATASET_HUB.
        """
        self.datasets = DATASET_HUB

    def show(self):
        """
        Prints a table of all datasets in the hub.
        """
        table = Table(title="Datasets", show_lines=True)

        table.add_column("Dataset Name", justify="left")

        for dataset in self.datasets:
            table.add_row(dataset.name)

        console = Console()
        console.print(table)

    def list(self):
        """
        Returns a list of all datasets in the hub.
        """
        return self.datasets

    def list_tags(self):
        """
        Returns a list of all tags in the hub.
        """
        all_tags = set()  # Using a set to automatically remove duplicates

        for dataset in self.datasets:
            all_tags.update(dataset.tags)

        return list(all_tags)

    def get(self, dataset_name):
        """
        Returns a Dataset object with the given name.

        Args:
            dataset_name (str): The name of the dataset to return.

        Raises:
            ValueError: If no dataset with the given name is found.
        """
        dataset = next((d for d in self.datasets if d.name == dataset_name), None)
        if dataset is None:
            raise ValueError(f"Dataset name '{dataset_name}' not found.")

        return dataset

    def get_by_tag(self, dataset_tag):
        """
        Returns a list of Dataset objects with the given tag.

        Args:
            dataset_name (str): Tag to query the datasets.

        Raises:
            ValueError: If no dataset with the given tag is found.
        """
        datasets = [d for d in self.datasets if dataset_tag in d.tags]
        if datasets is None:
            raise ValueError(f"Dataset with the tag '{dataset_tag}' not found.")

        return datasets

    def describe(self, dataset_name):
        """
        Prints a table of details for the given dataset.

        Args:
            dataset_name (str): The name of the dataset to describe.
        """
        dataset = self.get(dataset_name)

        table = Table(title=f"Details for {dataset_name}", show_lines=True)

        table.add_column("Attribute", justify="left")
        table.add_column("Value", justify="left")

        table.add_row("Path", dataset.path)
        table.add_row("Description", dataset.description)
        rendered_string = ", ".join(str(item) for item in dataset.tags)
        table.add_row("Tags", rendered_string)
        table.add_row("Documentation", dataset.documentation)

        console = Console()
        console.print(table)



import gcsfs
import polars as pl

from giza_datasets.constants import DATASET_HUB


class DatasetsLoader:
    """
    DatasetsLoader is a class for loading datasets from Google Cloud Storage (GCS).
    It uses the GCSFileSystem for accessing files and Polars for handling data.
    """

    def __init__(self):
        """
        Initializes the DatasetsLoader with a GCS filesystem and the dataset configuration.
        Verification is turned off for the GCS filesystem.
        """
        self.fs = gcsfs.GCSFileSystem(verify=False)
        self.dataset_hub = DATASET_HUB

    def _get_all_parquet_files(self, directory):
        """
        Recursively retrieves all the parquet file paths in the given directory.

        Args:
            directory: The GCS directory to search for parquet files.

        Returns:
            A list of full paths to all the parquet files found.
        """
        all_files = self.fs.ls(directory, detail=True)
        parquet_files = []

        for file_info in all_files:
            if file_info["type"] == "directory":
                parquet_files.extend(self._get_all_parquet_files(file_info["name"]))
            elif file_info["name"].endswith(".parquet"):
                full_path = "gs://" + file_info["name"]
                parquet_files.append(full_path)
        return parquet_files

    def _load_multiple_parquet_files(self, file_paths):
        """
        Loads multiple parquet files into a single Polars DataFrame.

        Args:
            file_paths: A list of file paths to load.

        Returns:
            A concatenated Polars DataFrame containing data from all files.
        """
        dfs = []
        for file_path in file_paths:
            with self.fs.open(file_path) as f:
                df = pl.read_parquet(f, use_pyarrow=True)
            dfs.append(df)
        concatenated_df = pl.concat(dfs, how="diagonal_relaxed")

        return concatenated_df

    def load(self, dataset_name):
        """
        Loads a dataset by name, either as a single file or multiple files.

        Args:
            dataset_name: The name of the dataset to load.

        Returns:
            A Polars DataFrame containing the loaded dataset.

        Raises:
            ValueError: If the dataset name is not found or if no parquet files are found.
        """
        gcs_path = None
        for dataset in self.dataset_hub:
            if dataset.name == dataset_name:
                gcs_path = dataset.path
                break

        if not gcs_path:
            raise ValueError(f"Dataset name '{dataset_name}' not found in Giza.")
        elif gcs_path.endswith(".parquet"):
            with self.fs.open(gcs_path) as f:
                df = pl.read_parquet(f, use_pyarrow=True)
        else:
            parquet_files = self._get_all_parquet_files(gcs_path)
            if not parquet_files:
                raise ValueError(
                    "No .parquet files were found in the directory or subdirectories."
                )
            df = self._load_multiple_parquet_files(parquet_files)
        return df



DEFILLAMA_SUPPORTED_PROJECTS = [
    "Lido",
    "Rocket-pool",
    "Binance-staked-eth",
    "Mantle-staked-eth",
    "Frax-ether",
    "Uniswap-v3",
    "Curve-dex",
    "Uniswap-v2",
    "Pancakeswap-amm",
    "Balancer-v2",
    "Pancakeswap-amm-v3",
    "Sushiswap",
    "Thorchain",
    "Convex-finance",
    "Stakestone",
    "Aura",
    "Pendle",
    "Coinwind",
    "Penpie",
    "Aave-v3",
    "Aave-v2",
    "Spark",
    "Compound-v3",
    "Compound-v2",
    "Morpho-aave",
    "Morpho-aavev3",
    "Benqi-lending",
    "Radiant-v2",
    "Yearn-finance",
    "Beefy",
    "Origin-ether",
    "Flamincome",
    "Sommelier",
]
